// Generated by CoffeeScript 1.10.0
(function() {
  var buildStack, createPaper, createSection, exit, generatePaper, getContent, json, options, page, ref, system, timeout, totalPages, vp, webpage;

  system = require('system');

  webpage = require('webpage');

  exit = function(error) {
    var message;
    if (typeof error === 'string') {
      message = error;
    }
    if (error) {
      system.stderr.write("html-pdf: " + (message || ("Unknown Error " + error)) + "\n");
    }
    return phantom.exit(error ? 1 : 0);
  };

  buildStack = function(msg, trace) {
    var msgStack;
    msgStack = [msg];
    if (trace != null ? trace.length : void 0) {
      msgStack.push('Stack:');
      trace.forEach(function(t) {
        return msgStack.push("  at " + (t.file || t.sourceURL) + ": " + t.line + " (in function " + t["function"] + ")");
      });
    }
    return msgStack.join('\n');
  };

  phantom.onError = function(msg, trace) {
    return exit(buildStack('Script - ' + msg, trace));
  };

  json = JSON.parse(system.stdin.readLine());

  if (!((ref = json.html) != null ? ref.trim() : void 0)) {
    exit('Did not receive any html');
  }

  options = json.options;

  page = webpage.create();

  if (options.httpHeaders) {
    page.customHeaders = options.httpHeaders;
  }

  page.content = json.html;

  if (vp = options.viewportSize) {
    page.viewportSize = vp;
  }

  totalPages = 0;

  page.onError = function(msg, trace) {
    return exit(buildStack('Evaluation - ' + msg, trace));
  };

  timeout = (options.timeout || 120000) + 2000;

  setTimeout(function() {
    return exit('Force timeout');
  }, timeout);

  getContent = function() {
    return page.evaluate(function() {
      var $body, body, f, footer, getElement, getElements, h, header, styles;
      getElements = function(doc, wildcard) {
        var $elem, $elements, elements, hasElements, i, j, len, match, wildcardMatcher;
        wildcardMatcher = new RegExp(wildcard + "(.*)");
        hasElements = false;
        elements = {};
        $elements = document.querySelectorAll("[id*='" + wildcard + "']");
        for (j = 0, len = $elements.length; j < len; j++) {
          $elem = $elements[j];
          if (match = $elem.attributes.id.value.match(wildcardMatcher)) {
            hasElements = true;
            i = match[1];
            elements[i] = $elem.outerHTML;
            $elem.parentNode.removeChild($elem);
          }
        }
        if (hasElements) {
          return elements;
        }
      };
      getElement = function(doc, id) {
        var $elem, html;
        if ($elem = doc.getElementById(id)) {
          html = $elem.outerHTML;
          $elem.parentNode.removeChild($elem);
          return html;
        }
      };
      styles = document.querySelectorAll('link,style');
      styles = Array.prototype.reduce.call(styles, (function(string, node) {
        return string + node.outerHTML;
      }), '');
      header = getElements(document, 'pageHeader-');
      footer = getElements(document, 'pageFooter-');
      h = getElement(document, 'pageHeader');
      f = getElement(document, 'pageFooter');
      if (h) {
        (header != null ? header : header = {})["default"] = h;
      }
      if (f) {
        (footer != null ? footer : footer = {})["default"] = f;
      }
      if ($body = document.getElementById('pageContent')) {
        body = $body.outerHTML;
      } else {
        body = document.body.outerHTML;
      }
      return {
        styles: styles,
        header: header,
        body: body,
        footer: footer
      };
    });
  };

  createPaper = function(options) {
    var paper;
    paper = {
      border: options.border || '0'
    };
    if (options.height && options.width) {
      paper.width = options.width;
      paper.height = options.height;
    } else {
      paper.format = options.format || 'A4';
      paper.orientation = options.orientation || 'portrait';
    }
    return paper;
  };

  createSection = function(section, content, options) {
    var c, o;
    c = content[section] || {};
    o = options[section] || {};
    return {
      height: o.height,
      contents: phantom.callback(function(pageNum, numPages) {
        var html;
        html = c[pageNum];
        if (pageNum === 1) {
          if (html == null) {
            html = c['first'];
          }
        }
        if (pageNum === numPages) {
          if (html == null) {
            html = c['last'];
          }
        }
        return (html || c["default"] || o.contents || '').replace('{{page}}', pageNum).replace('{{pages}}', numPages) + content.styles;
      })
    };
  };

  generatePaper = function(content, options) {
    var j, len, paper, ref1, ref2, ref3, section;
    paper = createPaper(options);
    ref1 = ['header', 'footer'];
    for (j = 0, len = ref1.length; j < len; j++) {
      section = ref1[j];
      if (options[section] || content[section]) {
        paper[section] = createSection(section, content, options);
      }
    }
    if ((ref2 = paper.header) != null) {
      if (ref2.height == null) {
        ref2.height = '46mm';
      }
    }
    if ((ref3 = paper.footer) != null) {
      if (ref3.height == null) {
        ref3.height = '28mm';
      }
    }
    return paper;
  };

  page.onLoadFinished = function(status) {
    var fileOptions, filename;
    page.paperSize = generatePaper(getContent(), options);
    fileOptions = {
      type: options.type || 'pdf',
      quality: options.quality || 75
    };
    filename = options.filename || ((options.directory || '/tmp') + "/html-pdf-" + system.pid + "." + fileOptions.type);
    page.render(filename, fileOptions);
    system.stdout.write(JSON.stringify({
      filename: filename
    }));
    return exit(null);
  };

}).call(this);
